#---------------------------------------------------------------
# Project         : LADS
# Module          : lads
# File            : ladsanomalies.py
# Version         : $Id: ladsanomalies.py,v 1.9 2005/01/02 22:26:48 fred Exp $
# Author          : Frederic Lepied
# Created On      : Mon Feb  3 08:27:58 2003
# Purpose         : Anomaly detection functions
#---------------------------------------------------------------

import socket
import string

from ladsconst import *
from ladsconfig import *
from ladslog import *

try:
    import GeoIP
    gi = GeoIP.new(GeoIP.GEOIP_MEMORY_CACHE)
    log('Using GeoIP')
except:
    GeoIP = None
    log('GeoIP not available')

def get_ip(h):
    if h == '':
        return 'local'
    try:
        return socket.gethostbyname(h)
    except:
        return h

def is_exception(excp, user, host, terminal):
    ip = get_ip(host)
    for e in excp:
        if e == host:
            return 1
        spl = string.split(e, '@')
        # match user and host
        l = len(spl)
        if l == 2 or l == 3:
            if l == 3:
                if not re.search('^' + spl[2] + '$', terminal):
                    continue
            if re.search('^' + spl[0] + '$', user):
                # exception is in the form of user@
                if spl[1] == '':
                    return 1
                # exception is in the form of user@regexp
                if re.search('^' + spl[1] + '$', host):
                    return 1
                # exception is in the form of user@host
                # and resolv host to allow one way resolution for dynamic
                # dns
                if ip == get_ip(spl[1]):
                    return 1
        # match host
        elif len(spl) == 1:
            # match as regexp
            if re.search('^' + spl[0] + '$', host):
                return 1
            # match as ip
            if get_ip(host) == get_ip(spl[0]):
                return 1
    return 0

def first_login_anomaly(entry, users, terms):
    if entry[UTMP_USER] not in users:
        return ('First login of %s' % entry[UTMP_USER], )
    else:
        return None

def multiple_login_anomaly(entry, users, terms):
    user = entry[UTMP_USER]
    excp = get_exception('foreign_login')

    if is_exception(excp, user, entry[UTMP_HOST], entry[UTMP_LINE]):
        return None
    
    for t in terms.values():
        if user != t[UTMP_USER]:
            continue
        
        if is_exception(excp, user, t[UTMP_HOST], t[UTMP_LINE]):
            continue
        
        if entry[UTMP_HOST] != t[UTMP_HOST]:
            return ('Multiple login of %s' % user, 'Logged from %s(%s) and %s(%s)' % (entry[UTMP_HOST], get_ip(entry[UTMP_HOST]), t[UTMP_HOST], get_ip(t[UTMP_HOST])))
    return None

def foreign_login_anomaly(entry, users, terms):
    host = entry[UTMP_HOST]
    user = entry[UTMP_USER]
    term = entry[UTMP_LINE]
    
    if is_exception(get_exception('foreign_login'), user, host, term):
        return None

    if host == '' or host == 'local':
        return ('Local login of %s on %s' % (user, term), )
    else:
        return ('Foreign login of %s from %s' % (user, host), 'Host address is %s' % get_ip(host))

def country_login_anomaly(entry, users, terms):

    if not GeoIP:
        return None
    
    host = entry[UTMP_HOST]
    user = entry[UTMP_USER]

    country = gi.country_code_by_name(host) or gi.country_code_by_addr(get_ip(host))

    if not country or "%s@%s" % (user, country) in get_exception('country_login'):
        return None

    return ('Strange country login of %s from %s' % (user, country), 'Host address is %s' % get_ip(host))

if __name__ == '__main__':
    import sys
    load_exceptions('/etc/security/lads.conf')
    excp = get_exception('foreign_login')
    print 'exception', excp
    if not is_exception(excp, sys.argv[1], sys.argv[2], sys.argv[3]):
        print 'anomaly'
    else:
        print 'valid'
        
# ladsanomalies.py ends here
