#---------------------------------------------------------------
# Project         : LADS
# Module          : lads
# File            : ladsutmp.py
# Version         : $Id: ladsutmp.py,v 1.2 2003/02/17 15:08:18 fred Exp $
# Author          : Frederic Lepied
# Created On      : Sat Feb 15 19:19:00 2003
# Purpose         : class to read utmp/wtmp entries
#---------------------------------------------------------------

import struct
import string
import os
import errno

import ladslog

WTMP_FILE = '/var/log/wtmp'

USER_PROCESS = 7
DEAD_PROCESS = 8

# Internal constants

_TOTAL = 384
_TYPE_OFFSET = 0
_TYPE_SIZE = 2
_TTY_OFFSET = 8
_TTY_SIZE = 32
_NAME_OFFSET = 44
_NAME_SIZE = 32
_HOST_OFFSET = 76
_HOST_SIZE = 256

# "384/8:32/44:32/76:256"

def remove_blank(str):
    idx = len(str)
    for loop in range(idx):
        if str[loop] == '\000':
            idx = loop
            break
    return str[:idx]

class utmpEntries:
    def __init__(self, filename = '/var/run/utmp'):
        self.fd = open(filename)
        self.data = ''
        
    def fileno(self):
        return self.fd.fileno()

    def get_entry(self):
        data = self.fd.read(_TOTAL)
        if data:
            self.data = self.data + data
            if len(self.data) != _TOTAL:
                return None
            type = struct.unpack('H', self.data[_TYPE_OFFSET:_TYPE_OFFSET + _TYPE_SIZE])[0]
            tty = remove_blank(struct.unpack('%ss' % _TTY_SIZE, self.data[_TTY_OFFSET:_TTY_OFFSET + _TTY_SIZE])[0])
            name = remove_blank(struct.unpack('%ss' % _NAME_SIZE, self.data[_NAME_OFFSET:_NAME_OFFSET + _NAME_SIZE])[0])
            hostname = remove_blank(struct.unpack('%ss' % _HOST_SIZE, self.data[_HOST_OFFSET:_HOST_OFFSET + _HOST_SIZE])[0])
            self.data = ''
            return (type, tty, name, hostname)
        else:
            return None

    def close(self):
        ladslog.log('closing utmp file')
        self.fd.close()
        
if __name__ == '__main__':
    entries = utmpEntries('/var/log/wtmp')
    e = entries.get_entry()
    while e:
        print e
        e = entries.get_entry()
    entries.close()
    
# ladsutmp.py ends here
